-- scope stuff
qualityoflife = qualityoflife or {};
local q = qualityoflife;

q.modules = q.modules or {};
q.modules["QoL_ClassicHeals"] = q.modules["QoL_ClassicHeals"] or {};
local c = q.modules["QoL_ClassicHeals"];

-- profiles
c.PROFILE_DEFAULT = "PROFILE_DEFAULT";
c.PROFILE_ADDON_LUNA = "PROFILE_ADDON_LUNA";
c.PROFILE_ADDON_GRID2 = "PROFILE_ADDON_GRID2";
c.PROFILE_ADDON_VUHDO = "PROFILE_ADDON_VUHDO";

-- sub profiles
c.PROFILE_PLAYER = "PROFILE_PLAYER";
c.PROFILE_TARGET = "PROFILE_TARGET";
c.PROFILE_TARGETTARGET = "PROFILE_TARGETTARGET";
c.PROFILE_PARTY = "PROFILE_PARTY";
c.PROFILE_RAID_COMPACT = "PROFILE_RAID_COMPACT";
c.PROFILE_RAID_PULLOUT = "PROFILE_RAID_PULLOUT";

c.PROFILEKEY_KEY = "PROFILEKEY_KEY";
c.PROFILEKEY_SUBPROFILES = "PROFILEKEY_SUBPROFILES";
c.PROFILEKEY_SUBPROFILES_ORDER = "PROFILEKEY_SUBPROFILES_ORDER";
c.PROFILEKEY_SPELLSOURCE = "PROFILEKEY_SPELLSOURCE";
c.PROFILEKEY_SPELLTYPE = "PROFILEKEY_SPELLTYPE";

local profiles;
local profileDescriptions = {
	[c.PROFILE_PLAYER] = "             Player Frame",
	[c.PROFILE_TARGET] = "             Target Frame",
	[c.PROFILE_TARGETTARGET] = "       Target-of-Target Frame",
	[c.PROFILE_PARTY] = "[2-5]    Party Frames",
	[c.PROFILE_RAID_COMPACT] = "[2-40] Compact Raid Frames",
	[c.PROFILE_RAID_PULLOUT] = "[2-40] Raid Pullout Groups",

	[c.PROFILE_DEFAULT] = "Default",
	[c.PROFILE_ADDON_LUNA] = "Addon: Luna Unit Frames",
	[c.PROFILE_ADDON_GRID2] = "Addon: Grid2",
	[c.PROFILE_ADDON_VUHDO] = "Addon: Vuhdo"
};

c.SPELLSOURCE_SELF = "SPELLSOURCE_SELF";
c.SPELLSOURCE_OTHERS = "SPELLSOURCE_OTHERS";

function c:GetProfiles(refresh)
	if not profiles or refresh then
		profiles = {};
		table.insert(profiles, {
			[c.PROFILEKEY_KEY] = c.PROFILE_DEFAULT;
			[c.PROFILEKEY_SUBPROFILES] = {
				[1] = c.PROFILE_PLAYER,
				[2] = c.PROFILE_TARGET,
				[3] = c.PROFILE_PARTY,
				[4] = c.PROFILE_RAID_COMPACT,
				[5] = c.PROFILE_RAID_PULLOUT,
				[6] = c.PROFILE_TARGETTARGET
			};
			[c.PROFILEKEY_SUBPROFILES_ORDER] = {
				[1] = c.PROFILE_PLAYER,
				[2] = c.PROFILE_TARGET,
				[3] = c.PROFILE_TARGETTARGET,
				[4] = c.PROFILE_PARTY,
				[5] = c.PROFILE_RAID_COMPACT,
				[6] = c.PROFILE_RAID_PULLOUT,
			};
		});

		if IsAddOnLoaded("Grid2") then
			table.insert(profiles, {
				[c.PROFILEKEY_KEY] = c.PROFILE_ADDON_GRID2;
				[c.PROFILEKEY_SUBPROFILES] = {
					[1] = c.PROFILE_RAID_COMPACT
				};
				[c.PROFILEKEY_SUBPROFILES_ORDER] = {
					[1] = c.PROFILE_RAID_COMPACT
				};
			});
		end

		if IsAddOnLoaded("LunaUnitFrames") then
			table.insert(profiles, {
				[c.PROFILEKEY_KEY] = c.PROFILE_ADDON_LUNA;
				[c.PROFILEKEY_SUBPROFILES] = {
					[1] = c.PROFILE_PLAYER,
					[2] = c.PROFILE_TARGET,
					[3] = c.PROFILE_PARTY,
					[4] = c.PROFILE_RAID_COMPACT
				};
				[c.PROFILEKEY_SUBPROFILES_ORDER] = {
					[1] = c.PROFILE_PLAYER,
					[2] = c.PROFILE_TARGET,
					[3] = c.PROFILE_PARTY,
					[4] = c.PROFILE_RAID_COMPACT
				};
			});
		end

		if IsAddOnLoaded("Vuhdo") then
			table.insert(profiles, {
				[c.PROFILEKEY_KEY] = c.PROFILE_ADDON_VUHDO;
				[c.PROFILEKEY_SUBPROFILES] = {
					[1] = c.PROFILE_PLAYER,
					[2] = c.PROFILE_TARGET,
					[3] = c.PROFILE_RAID_COMPACT
				};
				[c.PROFILEKEY_SUBPROFILES_ORDER] = {
					[1] = c.PROFILE_PLAYER,
					[2] = c.PROFILE_TARGET,
					[3] = c.PROFILE_RAID_COMPACT
				};
			});
		end
		c:InitProfiles();
	end
	return profiles;
end

function c:GetProfile(profileKey)
	for index, profileEntry in ipairs(c:GetProfiles()) do
		if profileEntry[c.PROFILEKEY_KEY] == profileKey then
			return profileEntry;
		end
	end
end

function c:InitProfiles(refresh)
	local profileNames = c:GetProfiles(refresh);

	local spellSources = {
		c.SPELLSOURCE_SELF,
		c.SPELLSOURCE_OTHERS
	};
	local spellTypes = {
		c.SPELL_HEAL,
		c.SPELL_PERIODIC_HEAL
	};

	-- default values
	for index, profile in ipairs(profileNames) do
		c:InitProfile(QOL_OPTIONS[c.name], profile, spellTypes, spellSources);
	end

	for _, spellType in ipairs(spellTypes) do
		for _, spellSource in ipairs(spellSources) do
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][spellType][spellSource][c.OPT_UIATTR_XOFFSET] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][spellType][spellSource][c.OPT_UIATTR_XOFFSET] or -41;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][spellType][spellSource][c.OPT_UIATTR_YOFFSET] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][spellType][spellSource][c.OPT_UIATTR_YOFFSET] or 6;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] or 16;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] or 20;

			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGET][spellType][spellSource][c.OPT_UIATTR_XOFFSET] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGET][spellType][spellSource][c.OPT_UIATTR_XOFFSET] or 41;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGET][spellType][spellSource][c.OPT_UIATTR_YOFFSET] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGET][spellType][spellSource][c.OPT_UIATTR_YOFFSET] or 6;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGET][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGET][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] or 16;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGET][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGET][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] or 20;

			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PARTY][spellType][spellSource][c.OPT_UIATTR_XOFFSET] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PARTY][spellType][spellSource][c.OPT_UIATTR_XOFFSET] or 0;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PARTY][spellType][spellSource][c.OPT_UIATTR_YOFFSET] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PARTY][spellType][spellSource][c.OPT_UIATTR_YOFFSET] or -2;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PARTY][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PARTY][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] or 16;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PARTY][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PARTY][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] or 20;

			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][spellType][spellSource][c.OPT_UIATTR_XOFFSET] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][spellType][spellSource][c.OPT_UIATTR_XOFFSET] or 0;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][spellType][spellSource][c.OPT_UIATTR_YOFFSET] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][spellType][spellSource][c.OPT_UIATTR_YOFFSET] or -2;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] or 12;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] or 16;

			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET][spellType][spellSource][c.OPT_UIATTR_XOFFSET] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET][spellType][spellSource][c.OPT_UIATTR_XOFFSET] or 0;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET][spellType][spellSource][c.OPT_UIATTR_YOFFSET] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET][spellType][spellSource][c.OPT_UIATTR_YOFFSET] or 0;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] or 12;
			QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] = QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] or 16;
		end
	end

	for index, profile in ipairs(profileNames) do
		c:InitProfileDefaults(QOL_OPTIONS[c.name], profile, spellTypes, spellSources);
	end
end

function c:InitProfile(table, profile, spellTypes, spellSources)
	local profileKey = profile[c.PROFILEKEY_KEY] or profile;
	table[profileKey] = table[profileKey] or {};
	if table[profileKey][c.OPT_UIATTR_ENABLED] == nil then
		table[profileKey][c.OPT_UIATTR_ENABLED] = true;
	end

	if not profile[c.PROFILEKEY_SUBPROFILES] then
		for _, spellType in ipairs(spellTypes) do
			table[profileKey][spellType] = table[profileKey][spellType] or {};
			if table[profileKey][spellType][c.OPT_UIATTR_ENABLED] == nil then
				table[profileKey][spellType][c.OPT_UIATTR_ENABLED] = true;
			end

			for _, spellSource in ipairs(spellSources) do
				table[profileKey][spellType][spellSource] = table[profileKey][spellType][spellSource] or {};
				if table[profileKey][spellType][spellSource][c.OPT_UIATTR_ENABLED] == nil and spellSource ~= c.SPELLSOURCE_OTHERS then
					-- disables spellsource.others by default
					table[profileKey][spellType][spellSource][c.OPT_UIATTR_ENABLED] = true;
				end

				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTFLAGS] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTFLAGS] or "OUTLINE";

				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR] or {};
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR]["r"] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR]["r"] or 0;
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR]["g"] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR]["g"] or 1;
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR]["b"] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR]["b"] or 0;
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR]["a"] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR]["a"] or 1;

				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_CRITICAL] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_CRITICAL] or {};
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_CRITICAL]["r"] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_CRITICAL]["r"] or 0;
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_CRITICAL]["g"] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_CRITICAL]["g"] or 1;
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_CRITICAL]["b"] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_CRITICAL]["b"] or 0;
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_CRITICAL]["a"] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_CRITICAL]["a"] or 1;

				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_OVERHEAL] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_OVERHEAL] or {};
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_OVERHEAL]["r"] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_OVERHEAL]["r"] or 0;
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_OVERHEAL]["g"] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_OVERHEAL]["g"] or 1;
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_OVERHEAL]["b"] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTCOLOR_OVERHEAL]["b"] or 0;

				if table[profileKey][spellType][spellSource][c.OPT_UIATTR_SHOWOVERHEAL] == nil then
					table[profileKey][spellType][spellSource][c.OPT_UIATTR_SHOWOVERHEAL] = true;
				end
			end
			table[profileKey][spellType][c.OPT_UI_LASTPROFILE] = table[profileKey][spellType][c.OPT_UI_LASTPROFILE] or spellSources[1];
		end
		table[profileKey][c.OPT_UI_LASTPROFILE] = table[profileKey][c.OPT_UI_LASTPROFILE] or spellTypes[1];
	else
		table[profileKey][c.OPT_UI_LASTPROFILE] = table[profileKey][c.OPT_UI_LASTPROFILE] or profile[c.PROFILEKEY_SUBPROFILES][1];
		for index, subProfile in ipairs(profile[c.PROFILEKEY_SUBPROFILES]) do
			c:InitProfile(table[profileKey], subProfile, spellTypes, spellSources);
		end
	end
end

function c:InitProfileDefaults(table, profile, spellTypes, spellSources)
	local profileKey = profile[c.PROFILEKEY_KEY] or profile;
	if not profile[c.PROFILEKEY_SUBPROFILES] then
		for _, spellType in ipairs(spellTypes) do
			for _, spellSource in ipairs(spellSources) do
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_XOFFSET] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_XOFFSET] or 0;
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_YOFFSET] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_YOFFSET] or -4;
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTSIZE] or 12;
				table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] = table[profileKey][spellType][spellSource][c.OPT_UIATTR_FONTSIZE_CRITICAL] or 16;
			end
		end
	else
		for index, subProfile in ipairs(profile[c.PROFILEKEY_SUBPROFILES]) do
			c:InitProfileDefaults(table[profileKey], subProfile, spellTypes, spellSources);
		end
	end
end

function c:GetProfileDescription(profileKey)
	return q:GetText(profileDescriptions[profileKey]);
end